#include <tizen.h>

#include "$(appName).h"

#include "conf.h"
#include "view.h"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *layout;
	Evas_Object *conform;
} appdata_s;

static struct {
	int win_width;
	int win_height;
} s_info = {
	.win_width = -1,
	.win_height = -1,
};

static void
__win_delete_request_cb(void *data, Evas_Object *obj, void *event_info);
static void
__layout_back_cb(void *data, Evas_Object *obj, void *event_info);
static void
__create_base_gui(appdata_s *ad);

static bool
__create_app(void *data);
static void
__control_app(app_control_h app_control, void *data);
static void
__pause_app(void *data);
static void
__resume_app(void *data);
static void
__terminate_app(void *data);

static void
__ui_app_lang_changed(app_event_info_h event_info, void *user_data);
static void
__ui_app_orient_changed(app_event_info_h event_info, void *user_data);
static void
__ui_app_region_changed(app_event_info_h event_info, void *user_data);
static void
__ui_app_low_battery(app_event_info_h event_info, void *user_data);
static void
__ui_app_low_memory(app_event_info_h event_info, void *user_data);

void
get_app_resource(const char *edj_file_in, char *edj_path_out, int edj_path_max)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(edj_path_out, edj_path_max, "%s%s", res_path,
			edj_file_in);
		free(res_path);
	}
}

int
get_app_win_width(void)
{
	return s_info.win_width;
}

int
get_app_win_height(void)
{
	return s_info.win_height;
}

int
main(int argc, char *argv[])
{
	appdata_s ad = {0,};
	int ret = 0;

	ui_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = __create_app;
	event_callback.terminate = __terminate_app;
	event_callback.pause = __pause_app;
	event_callback.resume = __resume_app;
	event_callback.app_control = __control_app;

	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_BATTERY],
		APP_EVENT_LOW_BATTERY, __ui_app_low_battery, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LOW_MEMORY],
		APP_EVENT_LOW_MEMORY, __ui_app_low_memory, &ad);
	ui_app_add_event_handler(
		&handlers[APP_EVENT_DEVICE_ORIENTATION_CHANGED],
		APP_EVENT_DEVICE_ORIENTATION_CHANGED,
		__ui_app_orient_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED],
		APP_EVENT_LANGUAGE_CHANGED,
		__ui_app_lang_changed, &ad);
	ui_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED],
		APP_EVENT_REGION_FORMAT_CHANGED,
		__ui_app_region_changed, &ad);
	ui_app_remove_event_handler(handlers[APP_EVENT_LOW_MEMORY]);

	ret = ui_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG,
			"ui_app_main() is failed. err = %d", ret);

	return ret;
}

static void
__win_delete_request_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static void
__layout_back_cb(void *data, Evas_Object *obj, void *event_info)
{
	appdata_s *ad = data;
	/* Let window go to hide state. */
	elm_win_lower(ad->win);
}

static void
__create_base_gui(appdata_s *ad)
{
	char edj_path[PATH_MAX] = {0, };
	Evas_Object *img_browser = NULL;
	Evas_Object *fav_browser = NULL;

	/* Window */
	ad->win = elm_win_util_standard_add(PACKAGE, PACKAGE);
	elm_win_conformant_set(ad->win, EINA_TRUE);
	elm_win_autodel_set(ad->win, EINA_TRUE);

	if (elm_win_wm_rotation_supported_get(ad->win)) {
		int rots[4] = { 0, 90, 180, 270 };
		elm_win_wm_rotation_available_rotations_set(ad->win,
			(const int *)(&rots), 4);
	}

	evas_object_smart_callback_add(ad->win, "delete,request",
		__win_delete_request_cb, NULL);
	elm_win_screen_size_get(ad->win, NULL, NULL, &s_info.win_width,
		&s_info.win_height);

	/* Conformant */
	ad->conform = elm_conformant_add(ad->win);
	elm_win_indicator_mode_set(ad->win, ELM_WIN_INDICATOR_SHOW);
	elm_win_indicator_opacity_set(ad->win, ELM_WIN_INDICATOR_OPAQUE);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	/* Base Layout */
	get_app_resource(EDJ_FILE, edj_path, (int)PATH_MAX);
	ad->layout = elm_layout_add(ad->win);
	elm_layout_file_set(ad->layout, edj_path, GRP_MAIN);
	evas_object_size_hint_weight_set(ad->layout, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	eext_object_event_callback_add(ad->layout, EEXT_CALLBACK_BACK,
		__layout_back_cb, ad);
	elm_object_content_set(ad->conform, ad->layout);

	/* Base Layout Content */
	img_browser = view_create_image_browser(ad->layout);
	if (!img_browser) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view_image_browser_create]");
		evas_object_del(ad->win);
		return;
	}
	elm_object_part_content_set(ad->layout, PART_IMG_BROWSER, img_browser);

	fav_browser = view_create_favorites_browser(ad->layout);
	if (!fav_browser) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][view_favorites_browser_create]");
		/*We must delete only window.
		All other object will be removed by window automatically*/
		evas_object_del(ad->win);
		return;
	}
	elm_object_part_content_set(ad->layout, PART_FAV_BROWSER, fav_browser);

	/* Show window after base gui is set up */
	evas_object_show(ad->layout);
	evas_object_show(ad->win);
}

static bool
__create_app(void *data)
{
	/* Hook to take necessary actions before main event loop starts
	   Initialize UI resources and application's data
	   If this function returns true, the main loop of application starts
	   If this function returns false, the application is terminated */
	appdata_s *ad = data;

	__create_base_gui(ad);

	return true;
}

static void
__control_app(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void
__pause_app(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void
__resume_app(void *data)
{
	/* Take necessary actions when application becomes visible. */
}

static void
__terminate_app(void *data)
{
	/* Release all resources. */
}

static void
__ui_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	system_settings_get_value_string(SYSTEM_SETTINGS_KEY_LOCALE_LANGUAGE,
		&locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
__ui_app_orient_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_DEVICE_ORIENTATION_CHANGED*/
	return;
}

static void
__ui_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

static void
__ui_app_low_battery(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_BATTERY*/
}

static void
__ui_app_low_memory(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LOW_MEMORY*/
}

