#include "$(appName).h"

#include "view.h"
#include "conf.h"
#include "data.h"

typedef enum {
	POPUP_MAX_ITEMS = 0,
	POPUP_ITEM_EXIST,
} popup_type_t;

typedef enum {
	ITEM_TYPE_GRID = 0,
	ITEM_TYPE_LIST,
} item_type_t;


static struct {
	Evas_Object *parent;
	Elm_Genlist_Item_Class *list_it_class;
	Elm_Gengrid_Item_Class *grid_it_class;
	int dx;
	int dy;
	Evas_Object *src_object;
	Elm_Object_Item *src_item;
	Evas_Object *src_container;
	int src_idx;
} s_info = {
	.parent = NULL,
	.list_it_class = NULL,
	.grid_it_class = NULL,
	.dx = -1,
	.dy = -1,
	.src_object = NULL,
	.src_item = NULL,
	.src_container = NULL,
	.src_idx = -1
};

/*======================= LOCAL FUNCTIONS - DEFINITIONS =====================*/
static void
__item_longpressed_cb(void *data, Evas_Object *obj, void *event_info);
static Evas_Object *
__get_container_item_content(Elm_Object_Item *it, const char *src_type);
static void
__show_popup(Evas_Object *parent, popup_type_t type);
static Evas_Object *
__create_container_item(Evas_Object *parent, const char *file,
	item_type_t item_type);

/*======================= GENLIST RELATED FUNCTIONS - DEFINITIONS ===========*/
static Evas_Object *
__get_genlist_content(void *data, Evas_Object *obj, const char *part);
static void
_del_genlist_item(void *data, Evas_Object *obj);
static Eina_Bool
__fill_genlist(Evas_Object *genlist);

/*======================= GENGRID RELATED FUNCTIONS - DEFINITIONS ===========*/
static Evas_Object *
__get_gengrid_content(void *data, Evas_Object *obj, const char *part);
static void
__gengrid_items_realized_cb(void *data, Evas_Object *obj, void *event_info);
static Eina_Bool
___gengrid_check_image_already_added(Eina_List *paths, const char *path);
static Eina_List *
__gengrid_get_image_paths(Evas_Object *gengrid);

/*======================= DRAG AND DROP RELATED FUNCTIONS ===================*/
static Eina_Bool
__genlist_dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev);
static Eina_Bool
__gengrid_dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev);
static Evas_Object *
__dnd_drag_icon_create_cb(void *data, Evas_Object *win,
				Evas_Coord *xoff, Evas_Coord *yoff);
static void
__dnd_drag_done_cb(void *data, Evas_Object *obj);



/*======================= PUBLIC FUNCTIONS IMPLEMENTATION ===================*/
Evas_Object *
view_create_image_browser(Evas_Object *parent)
{
	Evas_Object *genlist = NULL;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][genlist=NULL]");
		return NULL;
	}

	genlist = elm_genlist_add(parent);
	if (!genlist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][genlist=NULL]");
		return NULL;
	}

	if (!s_info.list_it_class) {
		s_info.list_it_class = elm_genlist_item_class_new();
		s_info.list_it_class->item_style = "full";
		s_info.list_it_class->func.content_get = __get_genlist_content;
		s_info.list_it_class->func.del = _del_genlist_item;
		s_info.list_it_class->func.text_get = NULL;
		s_info.list_it_class->func.state_get = NULL;
	}

	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL,
		EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);
	elm_genlist_select_mode_set(genlist, ELM_OBJECT_SELECT_MODE_NONE);

	if (!__fill_genlist(genlist)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][_genlist_fill]");
		evas_object_del(genlist);
		elm_genlist_item_class_free(s_info.list_it_class);
		s_info.list_it_class = NULL;
		return NULL;
	}

	evas_object_smart_callback_add(genlist, "longpressed",
		__item_longpressed_cb, NULL);

	elm_drop_target_add(genlist, ELM_SEL_FORMAT_TARGETS,
		NULL, NULL,
		NULL, NULL,
		NULL, NULL,
		__genlist_dnd_drop_cb, NULL);

	evas_object_show(genlist);

	return genlist;
}

Evas_Object *
view_create_favorites_browser(Evas_Object *parent)
{
	Evas_Object *gengrid = NULL;
	int width = -1;
	int height = -1;

	gengrid = elm_gengrid_add(parent);
	if (!gengrid) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][gengrid=NULL]");
		return NULL;
	}

	if (!s_info.grid_it_class) {
		s_info.grid_it_class = elm_gengrid_item_class_new();
		s_info.grid_it_class->item_style = "default";
		s_info.grid_it_class->func.content_get = __get_gengrid_content;
		s_info.grid_it_class->func.del = NULL;
		s_info.grid_it_class->func.text_get = NULL;
		s_info.grid_it_class->func.state_get = NULL;
	}

	elm_gengrid_select_mode_set(gengrid, ELM_OBJECT_SELECT_MODE_NONE);
	evas_object_size_hint_align_set(gengrid, EVAS_HINT_FILL,
		EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(gengrid, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);

	evas_object_smart_callback_add(gengrid, "longpressed",
		__item_longpressed_cb, NULL);
	evas_object_smart_callback_add(gengrid, "realized",
		__gengrid_items_realized_cb, NULL);

	elm_drop_target_add(gengrid, ELM_SEL_FORMAT_TARGETS,
		NULL, NULL,
		NULL, NULL,
		NULL, NULL,
		__gengrid_dnd_drop_cb, NULL);

	width = get_app_win_width();
	height = get_app_win_height();

	elm_gengrid_item_size_set(gengrid, width/MAX_FAVORITE_ITEMS,
		height * GENGRID_ITEM_HEIGHT_SCALE);

	evas_object_show(gengrid);

	s_info.parent = parent;
	return gengrid;
}

/*==================== GENLIST RELATED FUNCIONS - IMPLEMENTATION ============*/
static Evas_Object *
__get_genlist_content(void *data, Evas_Object *obj, const char *part)
{
	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA][data=NULL]");
		return NULL;
	}

	if (!strncmp(part, "elm.swallow.content", strlen(part)))
		return __create_container_item(obj, (char *) data,
			ITEM_TYPE_LIST);

	return NULL;
}

static void
_del_genlist_item(void *data, Evas_Object *obj)
{
	if (data)
		free(data);
}

static Eina_Bool
__fill_genlist(Evas_Object *genlist)
{
	Eina_List *l_img_paths = NULL;
	int i = 0;
	unsigned int count = 0;

	if (!genlist) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][genlist=NULL]");
		return EINA_FALSE;
	}

	l_img_paths = data_get_available_images();
	if (!l_img_paths) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][unable to load data]");
		return EINA_FALSE;
	}

	count = eina_list_count(l_img_paths);
	if (count == 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to get images count]");
		eina_list_free(l_img_paths);
		return EINA_FALSE;
	}

	for (i = 0 ; i < count; i++)
		elm_genlist_item_append(genlist, s_info.list_it_class,
			eina_list_nth(l_img_paths, i), NULL,
			ELM_GENLIST_ITEM_NONE, NULL, NULL);

	eina_list_free(l_img_paths);

	return EINA_TRUE;
}

/*======================= GENGRID RELATED FUNCTIONS - DEFINITIONS ===========*/
static Evas_Object *
__get_gengrid_content(void *data, Evas_Object *obj, const char *part)
{
	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA][data=NULL]");
		return NULL;
	}

	if (!strncmp(part, "elm.swallow.icon", strlen(part)))
		return (Evas_Object *) data;

	return NULL;
}

static void
__gengrid_items_realized_cb(void *data, Evas_Object *obj, void *event_info)
{
	data_update_favorites(__gengrid_get_image_paths(obj));
}

static Eina_List *
__gengrid_get_image_paths(Evas_Object *gengrid)
{
	Eina_List *items = NULL;
	Eina_List *it = NULL;
	Eina_List *paths = NULL;
	Elm_Object_Item *data = NULL;
	Evas_Object *image = NULL;
	const char *file = NULL;
	const char *group = NULL;

	if (!gengrid) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][gengrid=NULL]");
		return NULL;
	}

	items = elm_gengrid_realized_items_get(gengrid);
	if (!items) {
		dlog_print(DLOG_ERROR, LOG_TAG, "items=NULL");
		return NULL;
	}

	EINA_LIST_FOREACH(items, it, data)
	{
		if (!data)
			continue;

		image = __get_container_item_content(data, evas_object_type_get(gengrid));
		if (!image) {
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][image=NULL]");
			return NULL;
		}

		elm_image_file_get(image, &file, &group);
		paths = eina_list_append(paths, file);
	}

	return paths;
}

static Eina_Bool
___gengrid_check_image_already_added(Eina_List *paths, const char *path)
{
	Eina_List *it = NULL;
	char *compare_path = NULL;

	if (!paths || !paths) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][paths='%p'][path='%p']", paths, path);
		return EINA_FALSE;
	}

	EINA_LIST_FOREACH(paths, it, compare_path)
	{
		if (!compare_path)
			continue;

		if (!strncmp(compare_path, path, strlen(path)))
			return EINA_TRUE;
	}

	return EINA_FALSE;
}

/*============================= LOCAL FUNCIONS - IMPLEMENTATION =============*/
static Evas_Object *
__get_container_item_content(Elm_Object_Item *it, const char *src_type)
{
	Evas_Object *image = NULL;
	Evas_Object *box = NULL;
	Eina_List *box_content = NULL;
	char *content_part = NULL;

	if (!it || !src_type) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][it=NULL]");
		return NULL;
	}

	if (!strncmp(src_type, "elm_genlist", strlen(src_type))) {
		content_part = "elm.swallow.content";
	} else {
		content_part = "elm.swallow.icon";
	}

	box = elm_object_item_part_content_get(it, content_part);
	if (!box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_object_item_part_content_get]");
		return NULL;
	}

	box_content = elm_box_children_get(box);
	if (!box_content) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_box_children_get]");
		return NULL;
	}

	image = eina_list_nth(box_content, 0);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to get image]");
		return NULL;
	}

	return image;
}

static void
__item_longpressed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Elm_Object_Item *it = (Elm_Object_Item *) event_info;
	Evas_Object *image = __get_container_item_content(it, evas_object_type_get(obj));
	int mouse_x = -1;
	int mouse_y = -1;
	int evas_x = -1;
	int evas_y = -1;
	const char *file = NULL;

	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][failed to get image]");
		return;
	}

	evas_object_geometry_get(image, &mouse_x, &mouse_y, NULL, NULL);
	evas_pointer_output_xy_get(evas_object_evas_get(obj), &evas_x, &evas_y);
	s_info.src_object = image;
	s_info.src_container = obj;
	s_info.src_item = it;
	s_info.src_idx = elm_gengrid_item_index_get(it);

	elm_image_file_get(image, &file, NULL);
	elm_drag_start(image,
		ELM_SEL_FORMAT_TARGETS, file,
		ELM_XDND_ACTION_MOVE, __dnd_drag_icon_create_cb, image,
		NULL, NULL,
		NULL, NULL,
		__dnd_drag_done_cb, obj);

	elm_scroller_movement_block_set(obj,
		ELM_SCROLLER_MOVEMENT_BLOCK_VERTICAL);
}

static void
__show_popup(Evas_Object *parent, popup_type_t type)
{
	Evas_Object *popup = NULL;

	popup = elm_popup_add(s_info.parent);
	if (!popup) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_popup_add]");
		return;
	}

	switch (type) {
	case POPUP_ITEM_EXIST:
		elm_object_part_text_set(popup, "title,text",
			"Item already added");
		break;
	case POPUP_MAX_ITEMS:
		elm_object_part_text_set(popup, "title,text",
			"Maximum items in favorites");
		break;
	default:
		break;
	}

	elm_popup_timeout_set(popup, POPUP_TIMEOUT);
	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_CENTER);
	evas_object_show(popup);
}

static Evas_Object *
__create_container_item(Evas_Object *parent, const char *file,
	item_type_t item_type)
{
	Evas_Object *image = NULL;
	Evas_Object *box = NULL;
	int width = -1;
	int height = -1;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_PARAM][parent=NULL]");
		return NULL;
	}

	box = elm_box_add(parent);
	if (!box) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_box_add]");
		return NULL;
	}

	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);

	image = elm_image_add(parent);
	if (!image) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][image=NULL]");
		evas_object_del(box);
		return NULL;
	}

	if (!elm_image_file_set(image, file, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_image_file_set]");
		evas_object_del(image);
		evas_object_del(box);
		return NULL;
	}

	evas_object_size_hint_align_set(image, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_weight_set(image, EVAS_HINT_EXPAND,
		EVAS_HINT_EXPAND);


	width = get_app_win_width();
	height = get_app_win_height();

	switch (item_type) {
	case ITEM_TYPE_GRID:
		evas_object_size_hint_min_set(image,
			width/MAX_FAVORITE_ITEMS, height * GENGRID_ITEM_HEIGHT_SCALE);
		evas_object_size_hint_min_set(box,
			width/MAX_FAVORITE_ITEMS, height * GENGRID_ITEM_HEIGHT_SCALE);
		break;
	case ITEM_TYPE_LIST:
		evas_object_size_hint_min_set(box, width * GENLIST_ITEM_WIDTH_SCALE,
			height * GENLIST_ITEM_HEIGHT_SCALE);
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_TYPE]");
		return NULL;
	}

	elm_box_pack_end(box, image);

	evas_object_show(image);
	evas_object_show(box);

	return box;
}
/*============================= DRAG AND DROP RELATED FUNCTIONS =============*/
static Eina_Bool
__genlist_dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev)
{
	dlog_print(DLOG_INFO, LOG_TAG, "__genlist_dnd_drop_cb");
	int items_cnt = -1;

	if (obj != s_info.src_container) {
		elm_object_item_del(s_info.src_item);
		s_info.src_item = NULL;

		items_cnt = elm_gengrid_items_count(s_info.src_container);
		if (!items_cnt)
			elm_object_signal_emit(s_info.parent, SIGNAL_LABEL_SHOW,
				SIGNAL_SOURCE);
	}

	return EINA_TRUE;
}

static Eina_Bool
__gengrid_dnd_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *ev)
{
	dlog_print(DLOG_INFO, LOG_TAG, "__gengrid_dnd_drop_cb");

	int xposret = -1;
	int yposret = -1;
	Elm_Object_Item *rel_it = NULL;
	Eina_List *paths = NULL;
	const char *file = NULL;
	int x = -1;
	int y = -1;
	int dst_idx = -1;
	int count = -1;

	if (!ev) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA][ev=NULL]");
		return EINA_FALSE;
	}

	file = (char *) ev->data;
	if (!file) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[INVALID_CB_DATA][file_name=NULL]");
		return EINA_FALSE;
	}

	count = elm_gengrid_items_count(obj);
	if ((count == MAX_FAVORITE_ITEMS) && (obj != s_info.src_container)) {
		__show_popup(s_info.parent, POPUP_MAX_ITEMS);
		return EINA_FALSE;
	}

	evas_pointer_output_xy_get(evas_object_evas_get(obj), &x, &y);
	rel_it = elm_gengrid_at_xy_item_get(obj, x, y, &xposret, &yposret);

	/*Check if item is already added into the list*/
	if (count >= 1 && (obj != s_info.src_container)) {
		paths = __gengrid_get_image_paths(obj);
		if (!paths) {
			dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][_gengrid_get_image_paths]");
			return EINA_FALSE;
		}

		if (___gengrid_check_image_already_added(paths, file)) {
			dlog_print(DLOG_INFO, LOG_TAG, "[image is already added]");
			eina_list_free(paths);
			__show_popup(s_info.parent, POPUP_ITEM_EXIST);
			return EINA_FALSE;
		}
	}

	/*Genlist as source*/
	if ((s_info.src_container != obj) && !rel_it) {
		elm_gengrid_item_append(obj, s_info.grid_it_class,
			__create_container_item(obj, file, ITEM_TYPE_GRID),
			NULL, NULL);
	} else if ((s_info.src_container != obj) && rel_it) {
		elm_gengrid_item_insert_before(obj, s_info.grid_it_class,
			__create_container_item(obj, file, ITEM_TYPE_GRID),
			rel_it, NULL, NULL);
	} else if ((s_info.src_container == obj && rel_it)) {
		dst_idx = elm_gengrid_item_index_get(rel_it);
		elm_object_item_del(s_info.src_item);

		if (dst_idx > s_info.src_idx) {
			elm_gengrid_item_insert_after(obj, s_info.grid_it_class,
				__create_container_item(obj,
					file, ITEM_TYPE_GRID),
				rel_it, NULL, NULL);
		} else if (dst_idx < s_info.src_idx) {
			elm_gengrid_item_insert_before(obj,
				s_info.grid_it_class,
				__create_container_item(obj, file,
					ITEM_TYPE_GRID), rel_it, NULL, NULL);
		}
	}

	elm_object_signal_emit(s_info.parent, SIGNAL_LABEL_HIDE, SIGNAL_SOURCE);
	eina_list_free(paths);
	return EINA_TRUE;
}

static Evas_Object *
__dnd_drag_icon_create_cb(void *data, Evas_Object *win,
				Evas_Coord *xoff, Evas_Coord *yoff)
{
	dlog_print(DLOG_INFO, LOG_TAG, "__dnd_drag_icon_create_cb");
	Evas_Object *org_ic = (Evas_Object *) data;
	Evas_Object *dragged_ic = NULL;
	const char *file = NULL;
	const char *group = NULL;
	int x = -1;
	int y = -1;
	int w = -1;
	int h = -1;
	int xo = -1;
	int yo = -1;

	elm_image_file_get(org_ic, &file, &group);
	dragged_ic = elm_image_add(win);

	if (!elm_image_file_set(dragged_ic, file, group)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "[FAILED][elm_image_file_set]");
		return NULL;
	}

	evas_object_geometry_get(org_ic, &x, &y, &w, &h);
	evas_pointer_canvas_xy_get(evas_object_evas_get(org_ic), &xo, &yo);

	evas_object_resize(dragged_ic, w/2, h/2);
	evas_object_move(dragged_ic, x - w/2, y - h/2);

	/*setting offset to get center of moved icon behind mouse pointer*/
	if (xoff) *xoff = xo - w/4;
	if (yoff) *yoff = yo - h/4;

	return dragged_ic;
}

static void
__dnd_drag_done_cb(void *data, Evas_Object *obj)
{
	Evas_Object *src_obj = (Evas_Object *) data;

	if (src_obj)
		elm_scroller_movement_block_set(src_obj,
			ELM_SCROLLER_MOVEMENT_NO_BLOCK);
}

